import os, sys
import uuid
from PIL import Image, ImageOps
from multiprocessing.dummy import Pool
from multiprocessing import cpu_count


class Converter:

    def __init__(self, args):
        self.listDir = None
        self.root = None
        self.args = args

    def getDirs(self, root):
        self.root = root
        self.listDir = [os.path.join(root, o) for o in os.listdir(root)
                        if os.path.isdir(os.path.join(root, o))]
        return len(self.listDir)

    def readd(self, dir):
        for name in os.listdir(dir):
            path = os.path.join(dir, name)
            if os.path.isfile(path):
                self.convert(path)
            elif name != "output":
                self.readd(path)

    def convert(self, file):
        ext = os.path.splitext(file)
        img = Image.open(str(file))
        out_img = ImageOps.fit(img, (self.args[1], self.args[1]), Image.ANTIALIAS, 0, (0.5, 0.5))
        out_img = out_img.convert('RGB')

        dirname = os.path.split(file)[0] + '\\..\\output\\'

        if not os.path.exists(dirname):
            os.mkdir(dirname)

        out_file = dirname + uuid.uuid4().hex[:7].lower() + '.' + self.args[0]
        if self.args[0] == 'jpg':
            out_img.save(out_file, quality=100)
        else:
            out_img.save(out_file)
        print("[ 🌸 ] Converted ", out_file)



def convert_(*args):
    pool = Pool(cpu_count())
    converter = Converter(args)
    len_dirs = converter.getDirs(args[2])
    pool.map(converter.readd, converter.listDir) if len_dirs > 0 else converter.readd(converter.root)

    pool.close()
    pool.join()



import Augmentor

def augment_(args):

    p = Augmentor.Pipeline(args["root_dir"])

    #Default params
    probability_ = args["probability"]
    magnitude_ = args["magnitude"]
    left_r_ = args["left_r"]
    right_r = args["right_r"]
    grid_ = args["grid"]
    minf_ = args["min_factor"]
    maxf_ = args["max_factor"]
    num_ = args["num"]

    # Operations
    flip_ = args["flip"]
    rotate_ = args["rotate"]
    distortion_ = args["distortion"]
    skew_ = args["skew"]
    zoom_ = args["zoom"]

    if rotate_:
        p.rotate(probability=probability_, max_left_rotation=left_r_, max_right_rotation=right_r)
    if distortion_:
        p.random_distortion(probability=probability_, grid_width=grid_, grid_height=grid_, magnitude=magnitude_)
    if skew_:
        p.skew(probability=probability_, magnitude=0.7)
    if zoom_:
        p.zoom(probability=probability_, min_factor=minf_, max_factor=maxf_)
    if flip_:
        p.flip_random(probability=probability_)

    p.sample(num_)


from .utils import convert_
from .utils import augment_
import click


@click.group()
def cli():
    """
    Simple CLI for working with your CNN's dataset 🔥
    """
    pass


@cli.command()
@click.option('--type', '-t', 'type', help='output images extension', type=click.Choice(['jpg', 'png']), default='jpg', show_default=True)
@click.option('--target-size', '-s', 'size', help='output images size', default=128, show_default=True)
@click.argument('root_dir')
def convert(type, size, root_dir):
    """
    Converting images to specific size and format
    """
    convert_(type, size, root_dir)


@cli.command()
@click.option('--probability', '-p', 'probability', help='default probability for operations', default=0.5, show_default=True)
@click.option('--max-left-rotation', '-mlr', 'left_r', help='maximum left rotation (in degrees)', default=10, show_default=True)
@click.option('--max-right-rotation', '-mrr', 'right_r', help='maximum right rotation (in degrees)', default=10, show_default=True)
@click.option('--magnitude', '-mg', 'magnitude', help='magnitude value for distortion operation', default=8, show_default=True)
@click.option('--grid-width-height', '-gwh', 'grid', help='grid dimensions for distortion operation', default=8, show_default=True)
@click.option('--min-factor', '-minf', 'min_factor', help='min factor for scaling', default=1, show_default=True)
@click.option('--max-factor', '-maxf', 'max_factor', help='min factor for scaling', default=1.5, show_default=True)
@click.option('--num', '-n', 'num', help='number of output images after augmentation', default=500, show_default=True)

@click.option('--flip/--no-flip', default=True)
@click.option('--rotate/--no-rotate', default=True)
@click.option('--distortion/--no-distortion', default=True)
@click.option('--skew/--no-skew', default=True)
@click.option('--zoom/--no-zoom', default=True)

@click.argument('root_dir')
def augment(**args):
    """
    Dataset augmentation
    """
    augment_(args)
    pass





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='rudi',
    version='1.0',
    # scripts=['bin/rudi'],
    author="Petro Liashchynskyi",
    description="Small, fast and simple Python CLI image converter for CNNs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/liashchynskyi/rudi",
    packages=setuptools.find_packages(),
    install_requires=[
        'click',
        'Augmentor',
        'Pillow'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': ['rudi=rudi:cli'],
    }
)
